import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

/**
 * 环境类型
 */
enum EnvironmentType {
  /**
   * 开发、编辑环境
   */
  DevelopEditor,
  /**
   * 测试环境
   */
  Test,
  /**
   * 生产环境（中化内网中）
   */
  Production,

}

@Injectable({
  providedIn: 'root'
})
export class ObjectsService {

  static readonly c_apiRoot = "/api/Objects/";//普通上传的API根路径
  static readonly c_apiRoot_Multipart = "/api/";//分块上传的API根路径

  /**
   * 当前所处环境（打包前根据需要设置）
   */
  static currentEnvironmentType: EnvironmentType = EnvironmentType.DevelopEditor;

  /**
   * 根据当前环境，得到的地址头，包括固有部分 + 桶名
   */
  static baseUrl;

  /**
   * 组装完整路径
   * @param extensionPath 局部路径,也是序列化下来的那部分路径（从桶名的下一级开始）
   * 例子1（三维预案的模型）：  extensionPath = ceshi/assets/主体/outDoor/a.gltf   返回值：  /api/Objects/{桶名}/ceshi/assets/主体/outDoor/a.gltf，
   * 例子2（非三维部分的图片）：extensionPath = ceshi/web/a.rgb                    返回值：  /api/Objects/{桶名}/ceshi/web/a.rgb，                  
   */
  static getFullPath(extensionPath: string) {
    return ObjectsService.baseUrl + extensionPath;
  }

  /**
   * 根据所处环境不同，设置不同的桶名
   */
  obsRootByEnv = new Map<EnvironmentType, string>([
    [EnvironmentType.DevelopEditor, "sinochemweb/"], //开发与编辑
    [EnvironmentType.Test, "sinochemweb/"],//测试
    [EnvironmentType.Production, "sinochemweb/"],//生产环境（根据需要设置）最好是与测试环境的桶名相同
  ]);


  constructor(private http: HttpClient) {
    let bucketName = this.getBucketName();
    ObjectsService.baseUrl = ObjectsService.c_apiRoot + bucketName;
  }

  /**
   * 获取桶名
   */
  getBucketName() {

    return this.obsRootByEnv.get(ObjectsService.currentEnvironmentType);
  }

  /**
   * 普通上传，单个文件上限5M
   */
  postFile(extensionPath: string, file: File): Observable<Object> {
    let formData = new FormData()
    formData.append("file", file, file.name)
    let data = { keepOriginalName: 'true' }
    return this.http.post(ObjectsService.getFullPath(extensionPath), formData, { params: data });
  }

  //分块上传
  postFile_MultipartUpload(extensionPath: string, file: File): Promise<Object> {
    // let formData = new FormData()
    // formData.append("file", file, file.name)
    // return this.http.post(ObjectsService.baseUrl + extensionPath, formData);
    let data = { keepOriginalName: 'true', filename: file.name }
    return new Promise((resolve, reject) => {
      this.http.post(ObjectsService.c_apiRoot_Multipart + "NewMultipartUpload/" + this.getBucketName() + extensionPath, {}, { params: data }).subscribe(async (data: any) => { //初始化分段上传
        let instance = this;
        let objectName = data.objectName
        let uploadId = data.uploadId
        let PartNumberETag = []; //每次返回需要保存的信息
        //分块 处理
        let fileSize = file.size || null //上传文件的总大小
        let shardSize = 5 * 1024 * 1024 //5MB一个分片
        let allSlice = Math.ceil(fileSize / shardSize) //总文件/5MB===共分多少段

        for (let i = 0; i < allSlice; i++) { //循环分段上传
          let start = i * shardSize //切割文件开始位置
          let end = Math.min(fileSize, start + shardSize); //切割文件结束位置
          let formData = new FormData()
          formData.append("file", file.slice(start, end))

          //同步写法实现异步调用
          let result = await new Promise((resolve, reject) => {
            // await 需要后面返回一个 promise 对象
            this.http.post(ObjectsService.c_apiRoot_Multipart + `MultipartUpload/${instance.getBucketName()}${objectName}?uploadId=${uploadId}&partNumber=${i + 1}`, formData).subscribe((data: any) => {
              let msg = { "partNumber": data.partNumber || null, "eTag": data.eTag || null }
              resolve(msg) // 调用 promise 内置方法处理成功
            })
          });
          PartNumberETag.push(result)

          if (PartNumberETag.length === allSlice) { //分块上传完成
            let data = PartNumberETag
            let paramsData = { uploadId: uploadId };
            let path = ObjectsService.c_apiRoot_Multipart + `CompleteMultipartUpload/${instance.getBucketName()}` + objectName;
            this.http.post(path, data, { params: paramsData }).subscribe(data => {
              let objData: any = new Object();
              objData.fileName = file.name;
              // objData.filePath = (ObjectsService.baseUrl + objectName).replace(file.name, "");//桶名要根据当前环境动态组装
              objData.filePath = objectName;//此路径不全，前面缺少ObjectsService.baseUrl，这部分不可序列化，要在运行时拼接
              resolve(objData)
            })
          }
        }//for循环

        //分块 处理
      })
    })
  }
}



